<!DOCTYPE html>
<!--[if lt IE 7]>      <html class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html class="no-js lt-ie9 lt-ie8">        <![endif]-->
<!--[if IE 8]>         <html class="no-js lt-ie9">               <![endif]-->
<!--[if gt IE 8]><!--> <html class="no-js" lang="en">        <!--<![endif]-->
	<head>
		<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
		<meta charset="utf-8">
		<meta name="viewport" content="width=device-width">

		<title>
			OMNI{{? pagetitle }} - {{ pagetitle }}{{/ pagetitle }}
		</title>

		<link rel="stylesheet" href="/+assets/normalize.min.css">
		<link rel="stylesheet" href="/+assets/styles.css">

		<!--[if lt IE 9]>
		<script src="/+assets/html5respond.min.js"></script>
		<![endif]-->
	</head>
	<body>
		<div id="page-header-container">
			<header id="page-header">
				{{: header }}
				<div id="omni-logo">
					<h1>{{ pagetitle }}</h1>
					{{? realm }}
					<h3>Realm: <span class="realm-name">{{ realm }}</span></h3>
					{{/ realm }}
				</div>
				{{/ header }}
			</header>
		</div>

		<div id="page-content-container">
			{{? message }}
			<p class="message{{? error }} error {{/ error }}">
				{{? error }}<strong>Error:</strong> {{/ error }}{{ message }}
			</p>
			{{/ message }}
			{{{ content }}}
		</div>

		<div id="page-footer-container">
			<footer id="page-footer">
				{{: footer }}
				<p>
					OMNI — Copyright © 2014 Adrián Pérez
				</p>
				{{/ footer }}
			</footer>
		</div>
	</body>
</html>
