{{< content }}

<form method="POST" action="{{ form_url }}" class="single">
	<fieldset>
		<label for="username">User name</label>
		<input type="text" name="username" value="{{ username }}">
	</fieldset>

	<fieldset>
		<label for="password-old">Old Password</label>
		<input type="password" name="password-old" value="">
	</fieldset>

	<fieldset>
		<label for="password-new">New Password</label>
		<input type="password" name="password-new" value="">

		<label for="password-new-again">New Password (repeat)</label>
		<input type="password" name="password-new-again" value="">
	</fieldset>

	<input type="submit">
</form>
{{/ content }}

{{> master }}
